==> emelfm2-0.1.2.tar.gz: emelfm2-0.1.2/src/command/complete/e2_complete__path.c <==
/* $Id: e2_complete__path.c 1073 2005-08-24 02:59:20Z tpgww $
 * 
 * Copyright (C) 2004-2005 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "e2_complete.h"
#include <string.h>

/**
 * @brief get path directories
 * @return array of pointers to $PATH elements
 **/
static gchar **_e2_complete_path_get_path_dirs (void)
{
	const gchar *p = getenv ("PATH");  //_I( = this ok for all languages?
	gchar **ret = g_strsplit (p, ":", -1);
	return ret;
}
/**
 * @brief
 * @param dir
 * @param word
 * @param found pointer to list to record matching items
 * @param prefix
 * @return the number of successful matches
 **/
static gint _e2_complete_path_check_dir (gchar *dir, gchar *word,
	GList **found, gchar *prefix)
{
	printd (DEBUG, "check_dir (dir:%s,word:%s,found:_,prefix:%s)", dir, word, prefix);
	GError *error = NULL;
	//FIXME localise, vfs-ize
	GDir *d = g_dir_open (dir, 0, &error);
	if (error != NULL)
		return 0;
	gint ret = 0;
	gchar *cur = (gchar *) g_dir_read_name (d);	//CHECKME returns utf always ??
	while (cur != NULL)
	{
		if (g_str_has_prefix (cur, word))
		{
			gchar *tmp = g_strconcat (dir, G_DIR_SEPARATOR_S, cur, NULL);
			F_DECLARE;
			gchar *local = F_FILENAME_TO_LOCALE (tmp);
			gboolean is_exec = ! e2_fs_access (local, X_OK);
			gboolean is_dir = e2_fs_is_dir3 (local);
			F_FREE (tmp);
			if (is_exec || is_dir)
			{
				printd (DEBUG, "match: %s - %s", cur, word);
				gchar *utf = S_FILENAME_FROM_LOCALE (cur);
				if (is_dir)
				{
					gchar *temp = g_strconcat (utf, G_DIR_SEPARATOR_S, NULL);
					S_FREE (utf);
					utf = temp;
				}
				if (prefix == NULL)
					*found = g_list_append (*found, g_strdup(utf));
				else
				{
					gchar *f = g_strconcat (prefix, utf, NULL);
					*found = g_list_append (*found, f);
				}
				ret++;
				if (is_dir)	// if (is_dir) utf is _always_ a copy
					g_free (utf);
				else // if (!is_dir) utf is a copy if ndef FILES_UTF8ONLY
					S_FREE (utf);
			}
			g_free (tmp);
		}
		cur = (gchar *) g_dir_read_name (d);
	}

	g_dir_close (d);
	return ret;
}
/**
 * @brief create list of executable items that are valid completions of @a line
 *
 * The string to be completed extends from after the ' ' (if any) preceding
 * the cursor position @a pos, to that position. But if @a pos = 0, the string
 * will be empty
 * If the string starts with "./", the scanning occurs in the active directory
 * Otherwise the scan iterates over $PATH.
 *
 * @param line utf8 string containing the whole line to be completed
 * @param word utf8 string, copy of part of @a line with the 'word' to be completed
 * @param pos 0-based index of cursor position (characters, not bytes) in @a line
 * @param found pointer to list to record matching items
 * @param flags pointer to bit-flags indicating type of match (dirs or files)
 * @param data discretionary data pointer for this method
 * @param rt UNUSED pointer to data struct for the command line whose entry is to be completed
 * @return the number of successful matches
 **/
gint e2_complete_path (gchar *line, gchar *word, gint pos, GList **found,
	E2_CompleteFlags *flags, gpointer data, E2_CommandLineRuntime *rt)
{
	printd (DEBUG, "complete_path (line:%s,word:%s,pos:%d,found:_,flags:%d,data:_)",
		line, word, pos, *flags);

	if ((word == NULL) || (*word == '\0'))
		return 0;
	//the word to complete needs to be the whole line
	//(i.e. no ' ' in the line prior to the cursor position)
	//or else we can't match a path
	if (!g_str_equal (word, line))
	//error message in this case?
		return 0;

	gint ret = 0;

	if (g_str_has_prefix (word, E2_COMMAND_PREFIX))
	{	//current directory
		*flags |= E2_COMPLETE_FLAG_STOP;
		gchar *p1 = g_utf8_next_char (strrchr (word, G_DIR_SEPARATOR));
		gchar *p2 = e2_utf8_ndup (word, g_utf8_pointer_to_offset (word, p1));
		gchar *path = g_strconcat (app.pane_current->path, G_DIR_SEPARATOR_S, p2, NULL);
		ret += _e2_complete_path_check_dir (path, p1, found, p2);
		g_free (p2);
		g_free (path);
	}
	else
	{	//walk the PATH
		printd (DEBUG, "complete from path");
		gchar *word_locale = S_FILENAME_TO_LOCALE (word);
		gchar **paths = _e2_complete_path_get_path_dirs ();
		//iterate over the path members
		gint i = 0;
		while (paths[i] != NULL)
		{
			ret += _e2_complete_path_check_dir (paths[i], word_locale, found, NULL);
				i++;
		}
		g_strfreev (paths);
		S_FREE (word_locale);
	}

	if (ret > 0)
		*flags |= E2_COMPLETE_FLAG_STOP;
	return ret;
}

==> emelfm2-0.1.4.tar.gz: emelfm2-0.1.4/src/command/complete/e2_complete__path.c <==
/* $Id: e2_complete__path.c 1382 2005-11-02 21:20:57Z tpgww $
 * 
 * Copyright (C) 2004-2005 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "e2_complete.h"
#include <string.h>

/**
 * @brief get path directories
 * @return array of pointers to $PATH elements
 **/
static gchar **_e2_complete_path_get_path_dirs (void)
{
	const gchar *p = getenv ("PATH");  //_I( = this ok for all languages?
	gchar **ret = g_strsplit (p, ":", -1);
	return ret;
}
/**
 * @brief
 * @param dir
 * @param word
 * @param found pointer to list to record matching items
 * @param prefix
 * @return the number of successful matches
 **/
static gint _e2_complete_path_check_dir (gchar *dir, gchar *word,
	GList **found, gchar *prefix)
{
	printd (DEBUG, "check_dir (dir:%s,word:%s,found:_,prefix:%s)", dir, word, prefix);
	GError *error = NULL;
	//FIXME localise, vfs-ize
	GDir *d = g_dir_open (dir, 0, &error);
	if (error != NULL)
		return 0;
	gint ret = 0;
	gchar *cur = (gchar *) g_dir_read_name (d);	//CHECKME returns utf always ??
	while (cur != NULL)
	{
		if (g_str_has_prefix (cur, word))
		{
			gchar *tmp = g_strconcat (dir, G_DIR_SEPARATOR_S, cur, NULL);
			gchar *local = F_FILENAME_TO_LOCALE (tmp);
			gboolean is_exec = ! e2_fs_access (local, X_OK);
			gboolean is_dir = e2_fs_is_dir3 (local);
			F_FREE (tmp);
			if (is_exec || is_dir)
			{
				printd (DEBUG, "match: %s - %s", cur, word);
				gchar *utf = F_FILENAME_FROM_LOCALE (cur);
				if (is_dir)
				{
					gchar *temp = g_strconcat (utf, G_DIR_SEPARATOR_S, NULL);
					F_FREE (utf);
					utf = temp;
				}
				if (prefix == NULL)
					*found = g_list_append (*found, g_strdup(utf));
				else
				{
					gchar *f = g_strconcat (prefix, utf, NULL);
					*found = g_list_append (*found, f);
				}
				ret++;
				if (is_dir)	// if (is_dir) utf is _always_ a copy
					g_free (utf);
				else // if (!is_dir) utf is a copy if ndef FILES_UTF8ONLY
					F_FREE (utf);
			}
			g_free (tmp);
		}
		cur = (gchar *) g_dir_read_name (d);
	}

	g_dir_close (d);
	return ret;
}
/**
 * @brief create list of executable items that are valid completions of @a line
 *
 * The string to be completed extends from after the ' ' (if any) preceding
 * the cursor position @a pos, to that position. But if @a pos = 0, the string
 * will be empty
 * If the string starts with "./", the scanning occurs in the active directory
 * Otherwise the scan iterates over $PATH.
 *
 * @param line utf8 string containing the whole line to be completed
 * @param word utf8 string, copy of part of @a line with the 'word' to be completed
 * @param pos 0-based index of cursor position (characters, not bytes) in @a line
 * @param found pointer to list to record matching items
 * @param flags pointer to bit-flags indicating type of match (dirs or files)
 * @param data discretionary data pointer for this method
 * @param rt UNUSED pointer to data struct for the command line whose entry is to be completed
 * @return the number of successful matches
 **/
gint e2_complete_path (gchar *line, gchar *word, gint pos, GList **found,
	E2_CompleteFlags *flags, gpointer data, E2_CommandLineRuntime *rt)
{
	printd (DEBUG, "complete_path (line:%s,word:%s,pos:%d,found:_,flags:%d,data:_)",
		line, word, pos, *flags);

	if ((word == NULL) || (*word == '\0'))
		return 0;
	//the word to complete needs to be the whole line
	//(i.e. no ' ' in the line prior to the cursor position)
	//or else we can't match a path
	if (!g_str_equal (word, line))
	//error message in this case?
		return 0;

	gint ret = 0;

	if (g_str_has_prefix (word, E2_COMMAND_PREFIX))
	{	//current directory
		*flags |= E2_COMPLETE_FLAG_STOP;
		gchar *p1 = g_utf8_next_char (strrchr (word, G_DIR_SEPARATOR));
		gchar *p2 = e2_utf8_ndup (word, g_utf8_pointer_to_offset (word, p1));
		gchar *path = g_strconcat (app.pane_current->path, G_DIR_SEPARATOR_S, p2, NULL);
		ret += _e2_complete_path_check_dir (path, p1, found, p2);
		g_free (p2);
		g_free (path);
	}
	else
	{	//walk the PATH
		printd (DEBUG, "complete from path");
		gchar *word_locale = F_FILENAME_TO_LOCALE (word);
		gchar **paths = _e2_complete_path_get_path_dirs ();
		//iterate over the path members
		gint i = 0;
		while (paths[i] != NULL)
		{
			ret += _e2_complete_path_check_dir (paths[i], word_locale, found, NULL);
				i++;
		}
		g_strfreev (paths);
		F_FREE (word_locale);
	}

	if (ret > 0)
		*flags |= E2_COMPLETE_FLAG_STOP;
	return ret;
}
